#include <Wire.h>


void skanujI2c() {
  byte error, address;
  int nDevices;

  Serial.println("Scanning...");

  nDevices = 0;
  for (address = 0; address < 127; address++ )
  {
    // The i2c_scanner uses the return value of
    // the Write.endTransmisstion to see if
    // a device did acknowledge to the address.
    Wire.beginTransmission(address);
    error = Wire.endTransmission();

    if (error == 0)
    {
      Serial.print("I2C device found at address 0x");
      if (address < 16)
        Serial.print("0");
      Serial.print(address, HEX);
      Serial.println("  !");

      nDevices++;
    }
    else if (error == 4)
    {
      Serial.print("Unknown error at address 0x");
      if (address < 16)
        Serial.print("0");
      Serial.println(address, HEX);
    }
  }
  if (nDevices == 0)
    Serial.println("No I2C devices found\n");
  else
    Serial.println("done\n");
}


//=========================================================//
void setup() {
  Serial.begin(115200);
  Wire.begin(); // Initiate the Wire library
  Wire.setClock( 40000 );
  Serial.println( "*****START *****\n\r" );


  skanujI2c();
}


//=========================================================//
void loop() {
  char txt[50];


  uint8_t static rejestr = 0x30;
  uint16_t static cnt;
  sprintf( txt, "\nSend %d(%x)", ++cnt, rejestr ); Serial.print( txt );
  Wire.beginTransmission( 0x38 );
  Wire.write( rejestr );  // Adr H
  Wire.write( 't' );
  Wire.write( 's' );
  Wire.write( 't' );
  uint8_t st = Wire.endTransmission();  // True jeśli chcemy aby transmisja zakończyła się sygnałem STOP.
  // False jeśli tego nie chcemy. Możemy nic nie wpisywać, domyślna wartość to true.
  // st =
  //    0 – Sukces
  //    1 – Bufor danych wysyłanych uległ przepełnieniu
  //    2 – Urządzenie podrzędne nie potwierdziło odbioru adresu (NACK)
  //    3 – Urządzenie podrzędne nie potwierdziło odbioru danych (NACK)
  //    4 – Wystąpił inny błąd.
  sprintf( txt, " End=%d\r", st ); Serial.println( txt );


  delay(5); // INFO: WAZNE: Pomiędzy transmisjami 5ms pauzy


  Serial.print( "Read..'" );
#define TAB_LEN_MAX  50
  uint8_t static tab[ TAB_LEN_MAX ];
  uint8_t tabPtr = 0;
  Wire.requestFrom( 0x38, TAB_LEN_MAX );    // request 0x30 bytes from slave device $38
  while (Wire.available())   // slave may send less than requested
  {
    uint8_t c = tab[ tabPtr ] = Wire.read(); // receive a byte as character
    if ( tabPtr < TAB_LEN_MAX ) tabPtr++;
    Serial.print(c, HEX); Serial.print(' ');        // print the character
  }
  Serial.println( "' End." );

  union t32_8 {
    uint32_t t_u32;
    uint8_t t_u8_t[8];
    float t_float;
    double t_double;
  };
  union t32_8 dat;

enum{ FGEN_REJ_ID, FGEN_REJ_UTC, FGEN_REJ_PERIOD_U32, FGEN_REJ_PERIOD_FLOAT, FGEN_REJ_PERIOD_DOUBLE, FGEN_REJ_DELTA_FLOAT, FGEN_REJ_DELTA_DOUBLE,
  FGEN_REJ_END };
  if ( (rejestr & 15) == FGEN_REJ_ID ) {
    sprintf( txt, "* ID1: %s", tab ); Serial.print( txt );
    sprintf( txt, " v%x.%02x", tab[13], tab[12] ); Serial.print( txt );
    sprintf( txt, " %s", tab+14 ); Serial.println( txt );
  
    //sprintf( txt, "Date %s", tab+30 ); Serial.print( txt );
    //sprintf( txt, " %s", tab+42 ); Serial.println( txt ); 
  }
  else if ( (rejestr & 15) == FGEN_REJ_UTC ) {
    sprintf( txt, "* UTC: %c%c:%c%c:%c%c (%d) last=%c%c:%c%c:%c%c", tab[1], tab[2], tab[3], tab[4], tab[5], tab[6], tab[0], tab[7], tab[8], tab[9], tab[10], tab[11], tab[12] ); Serial.println( txt );
  }
  else if ( (rejestr & 15) == FGEN_REJ_PERIOD_U32 ) {
    uint16_t dataL = tab[ 0 ] + (tab[ 1 ] << 8);
    uint16_t dataH = tab[ 2 ] + (tab[ 3 ] << 8);
    uint32_t data = dataL + (dataH * 65536);
    Serial.print( "* Period=0x" ); Serial.print( data, HEX ); Serial.print( " #" ); Serial.println( data );
  }
  else if ( (rejestr & 15) == FGEN_REJ_PERIOD_FLOAT ) {
    memcpy( &dat.t_u8_t, tab, 8);
    Serial.print( "* Period f=" ); Serial.println( dat.t_float );
  }
  //else if ( (rejestr & 15) == FGEN_REJ_PERIOD_DOUBLE ) {
  //  memcpy( &dat.t_u8_t, tab, 8);
  //  Serial.print( "* Period d=" ); Serial.println( dat.t_double );
  //}
  else if ( (rejestr & 15) == FGEN_REJ_DELTA_FLOAT ) {
    memcpy( &dat.t_u8_t, tab, 8);
    Serial.print( "* Delta f=" ); Serial.println( dat.t_float );
  }
  //else if ( (rejestr & 15) == FGEN_REJ_DELTA_DOUBLE ) {
  //  memcpy( &dat.t_u8_t, tab, 8);
  //  Serial.print( "* Delta d=" ); Serial.println( dat.t_double );
  //}

  if ( ++rejestr >= 0x36 ) rejestr = 0x30;
  if ( (rejestr % 5) == 0 )Serial.println( "===================================" );
  delay( 2500 );
}
